/** 
 * Copyright (c) 2012, Regents of the University of Colorado 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * Neither the name of the University of Colorado at Boulder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE. 
 */
package org.cleartk.classifier.feature.transform;

import java.util.ArrayList;
import java.util.List;
import java.util.Stack;

import org.cleartk.classifier.Feature;
import org.cleartk.classifier.Instance;

/**
 * Copyright (c) 2012, Regents of the University of Colorado <br>
 * All rights reserved.
 */

public abstract class TrainableExtractor_ImplBase<OUTCOME_T> implements
    TrainableExtractor<OUTCOME_T> {

  protected String name;

  public TrainableExtractor_ImplBase(String name) {
    this.name = name;
  }

  protected boolean isTransformable(Feature feature) {
    return feature instanceof TransformableFeature && this.name.equals(feature.getName());
  }

  /**
   * Recursively look through all features in an instance to find the features this extractor is
   * responsible for extracting
   */
  protected List<TransformableFeature> selectTransformableFeatures(Instance<OUTCOME_T> instance) {

    List<TransformableFeature> features = new ArrayList<TransformableFeature>();
    Stack<Feature> featuresToProcess = new Stack<Feature>();
    featuresToProcess.addAll(instance.getFeatures());

    while (!featuresToProcess.isEmpty()) {
      Feature currFeature = featuresToProcess.pop();
      if (currFeature instanceof TransformableFeature) {
        TransformableFeature transformable = (TransformableFeature) currFeature;
        if (transformable.getName() == this.name) {
          features.add(transformable);
        } else {
          featuresToProcess.addAll(transformable.getFeatures());
        }
      }
    }
    return features;
  }

}
